/*==============================================================================
    FILE:           sys_services_uart.c

    DESCRIPTION:    Uses ADI System Services to interface with a BF707 UART.

    Copyright (c) 2014 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/
#include "sys_services_uart.h"
#include <drivers/uart/adi_uart.h>
#include <services/pwr/adi_pwr.h>
#include <services/tmr/adi_tmr.h>
#include <stddef.h>
#include <ADSP-BF707_device.h>

#include "user_cdc.h"

#define SYS_SERVICES_UART_NUM       1

#define CLKIN                   (25 * (1000000))
#define CCLK_FREQ               (400 * (1000000))
#define SCLK_FREQ               (200 * (1000000))


/* UART driver memory */
uint8_t sys_services_uart_driver_memory[ADI_UART_BIDIR_INT_MEMORY_SIZE];
/* UART driver handle */
ADI_UART_HANDLE sys_services_uart_hdevice;

/* Set to true when the UART TX is idle (not transmitting). */
volatile bool sys_services_uart_tx_idle = true;

/* Serial data RX buffer.
   The Buffer size is set to 1 so the ADI_UART_EVENT_RX_BUFFER_PROCESSED event
   is called for every byte. */
#define SYS_SERVICES_UART_RX_BUFFER_SIZE        1
static unsigned char sys_services_uart_rx_buffer[SYS_SERVICES_UART_RX_BUFFER_SIZE];

/* Variables used to transmit a Break condition. */
static CLD_Time sys_services_uart_break_time = 0;
static CLD_Time sys_services_uart_break_timeout = 0;
static bool sys_services_break_active = false;

static void sys_services_uart_irq_callback(void* pHandle, uint32_t u32Arg, void* pArg);

/*=============================================================================
Function:       sys_services_uart_init

Parameters:     None.

Description:    Initializes the ADI System Services UART interface..

Returns:        None.
==============================================================================*/
void sys_services_uart_init (void)
{
    ADI_PWR_RESULT pwr_result;
    ADI_UART_RESULT uart_result;

    /* Configure the UART 1 TX and RX pins. */
    pADI_PORTC->DIR_SET = (1 << 0);
    /* Enable pin function */
    pADI_PORTC->FER_SET = (0x03 << 0);
    /* Set pin mux */
    pADI_PORTC->MUX &= ~(0x0f << 0);

    adi_pwr_Init(0, CLKIN);
    pwr_result = adi_pwr_SetPowerMode(0, ADI_PWR_MODE_FULL_ON);
    if (pwr_result == ADI_PWR_SUCCESS)
    {
        pwr_result = adi_pwr_SetFreq(0, CCLK_FREQ, SCLK_FREQ);
        if (pwr_result == ADI_PWR_SUCCESS)
        {
            uart_result = adi_uart_Open(SYS_SERVICES_UART_NUM, ADI_UART_DIR_BIDIRECTION, sys_services_uart_driver_memory, ADI_UART_BIDIR_INT_MEMORY_SIZE, &sys_services_uart_hdevice);
            if (uart_result == ADI_UART_SUCCESS)
            {
                uart_result = adi_uart_SetMode(sys_services_uart_hdevice, ADI_UART_MODE_UART);
                if (uart_result == ADI_UART_SUCCESS)
                {
                    uart_result = adi_uart_EnableDMAMode(sys_services_uart_hdevice, false);
                    if (uart_result == ADI_UART_SUCCESS)
                    {
                        /* register the callback function */
                        uart_result = adi_uart_RegisterCallback(sys_services_uart_hdevice, sys_services_uart_irq_callback, (void*)0);
                        if (uart_result == ADI_UART_SUCCESS)
                        {
                            uart_result = adi_uart_EnableLineStatusEvents(sys_services_uart_hdevice, true);
                            if (uart_result != ADI_UART_SUCCESS)
                            {
                                cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART enable line status error 0x%x\n\r", uart_result);
                            }
                        }
                        else
                        {
                            cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART register callback error 0x%x\n\r", uart_result);
                        }
                    }
                    else
                    {
                        cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART set DMA mode error 0x%x\n\r", uart_result);
                    }
                }
                else
                {
                    cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART set mode error 0x%x\n\r", uart_result);
                }
            }
            else
            {
                cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART open error 0x%x\n\r", uart_result);
            }
        }
        else
        {
            cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services set freq error 0x%x\n\r", pwr_result);
        }
    }
    else
    {
        cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services set power mode error 0x%x\n\r", pwr_result);
    }
}

/*=============================================================================
Function:       sys_services_uart_main

Parameters:     None.

Description:    System Services UART mainline function.

Returns:        None.
==============================================================================*/
void sys_services_uart_main (void)
{
    /* If the CDC Host requested RS-232 Break, and the requested duration wasn't 0xffff
       (A duration of 0xFFFF sets the break condition until commanded to stop by
        the Host). */
    if ((sys_services_break_active == true) && (sys_services_uart_break_timeout != 0xffff))
    {
        if (cld_time_passed_ms(sys_services_uart_break_time) > sys_services_uart_break_timeout)
        {
            adi_uart_ForceTxBreak(sys_services_uart_hdevice, false);
            sys_services_break_active = false;
        }
    }
}

/*=============================================================================
Function:       sys_services_uart_set_line_coding

Parameters:     p_config - Pointer to the requested CDC Line Coding parameters.

Description:    Configures the UART using the CDC Line Coding parameter passed to
                the function.

Returns:        CLD_SUCCESS - UART configuration successful.
                CLD_FAIL - UART configuration failed.
==============================================================================*/
CLD_RV sys_services_uart_set_line_coding (CLD_BF70x_CDC_Line_Coding * p_config)
{
    ADI_UART_RESULT uart_result;
    ADI_UART_STOPBITS num_stop_bits;
    ADI_UART_WORDLEN word_length;
    ADI_UART_PARITY parity;
    CLD_RV rv = CLD_SUCCESS;

    /* Convert the requested CDC number of stop bits to the equivalent ADI System
       Services number of stop bits setting */
    switch (p_config->num_stop_bits)
    {
        case CLD_BF70x_CDC_LINE_CODING_1_STOP_BITS:
            num_stop_bits = ADI_UART_ONE_STOPBIT;
        break;
        case CLD_BF70x_CDC_LINE_CODING_1_5_STOP_BITS:
        num_stop_bits = ADI_UART_ONE_AND_HALF_STOPBITS;
        break;
        case CLD_BF70x_CDC_LINE_CODING_2_STOP_BITS:
        num_stop_bits = ADI_UART_TWO_STOPBITS;
        break;
        default:
            cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "Invalid num stop bits\n\r");
            rv = CLD_FAIL;
        break;
    }
   /* Convert the requested CDC number of data bits to the equivalent ADI System
       Services word length setting */
    switch (p_config->num_data_bits)
    {
        case 5:
            word_length = ADI_UART_WORDLEN_5BITS;
        break;
        case 6:
            word_length = ADI_UART_WORDLEN_6BITS;
        break;
        case 7:
            word_length = ADI_UART_WORDLEN_7BITS;
        break;
        case 8:
            word_length = ADI_UART_WORDLEN_8BITS;
        break;
        default:
            cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "Invalid word length\n\r");
            rv = CLD_FAIL;
        break;
    }

    /* Convert the requested CDC parity setting to the equivalent ADI System
       Services parity setting */
    switch (p_config->parity)
    {
        case CLD_BF70x_CDC_LINE_CODING_PARITY_NONE:
            parity = ADI_UART_NO_PARITY;
        break;
        case CLD_BF70x_CDC_LINE_CODING_PARITY_ODD:
            parity = ADI_UART_ODD_PARITY;
        break;
        case CLD_BF70x_CDC_LINE_CODING_PARITY_EVEN:
            parity = ADI_UART_EVEN_PARITY;
        break;
        default:
            cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "Invalid parity\n\r");
            rv = CLD_FAIL;
        break;
    }

    if (rv != CLD_FAIL)
    {
        /* Disable the UART TX, RX and Line Status before re-configuring the UART. */
        uart_result = adi_uart_EnableTx(sys_services_uart_hdevice, false);
        uart_result = adi_uart_EnableRx(sys_services_uart_hdevice, false);
        uart_result = adi_uart_EnableLineStatusEvents(sys_services_uart_hdevice, false);

        uart_result =  adi_uart_SetBaudRate(sys_services_uart_hdevice, p_config->data_terminal_rate);
        if (uart_result == ADI_UART_SUCCESS)
        {
            uart_result = adi_uart_SetNumStopBits(sys_services_uart_hdevice, num_stop_bits);
            if (uart_result == ADI_UART_SUCCESS)
            {
                uart_result = adi_uart_SetWordLen(sys_services_uart_hdevice, word_length);
                if (uart_result == ADI_UART_SUCCESS)
                {
                    uart_result = adi_uart_SetParity(sys_services_uart_hdevice, parity);
                    if (uart_result == ADI_UART_SUCCESS)
                    {
                        uart_result = adi_uart_SubmitRxBuffer(sys_services_uart_hdevice, sys_services_uart_rx_buffer, SYS_SERVICES_UART_RX_BUFFER_SIZE);

                        /* Re-enable the UART RX and Line Status events. */
                        uart_result = adi_uart_EnableRx(sys_services_uart_hdevice, true);
                        uart_result = adi_uart_EnableLineStatusEvents(sys_services_uart_hdevice, true);
                    }
                    else
                    {
                        cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART set parity error 0x%x\n\r", uart_result);
                        rv = CLD_FAIL;
                    }
                }
                else
                {
                    cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART set word length error 0x%x\n\r", uart_result);
                    rv = CLD_FAIL;
                }
            }
            else
            {
                cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART set num stop bits error 0x%x\n\r", uart_result);
                rv = CLD_FAIL;
            }
        }
        else
        {
            cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "ADI Sys Services UART set baud rate error 0x%x\n\r", uart_result);
            rv = CLD_FAIL;
        }
    }
    return rv;
}

/*=============================================================================
Function:       sys_services_uart_tx_buffer

Parameters:     length - number of bytes to transmit.
                p_buffer - pointer to the data buffer to transmit

Description:    Transmits the specified data to the ADI System Services controlled
                UART.

Returns:        CLD_SUCCESS - Data transmission started successfully.
                CLD_FAIL - Data transmission failed.
==============================================================================*/
CLD_RV sys_services_uart_tx_buffer (unsigned short length, unsigned char * p_buffer)
{
    ADI_UART_RESULT uart_result;
    if (length <= ADI_UART_BIDIR_DMA_MEMORY_SIZE)
    {
        uart_result = adi_uart_SubmitTxBuffer(sys_services_uart_hdevice, p_buffer, length);

        /* enable the UART transfer */
        uart_result = adi_uart_EnableTx(sys_services_uart_hdevice, true);
        sys_services_uart_tx_idle = false;

        return CLD_SUCCESS;
    }
    else
    {
        return CLD_FAIL;
    }
}

/*=============================================================================
Function:       sys_services_uart_send_break

Parameters:     duration - requested break duration in milliseconds.

Description:    Transmits the RS-232 Break condition.

Returns:        None.
==============================================================================*/
void sys_services_uart_send_break (unsigned short duration)
{
    if (duration == 0)
    {
        /* Disable the Break condition. */
        adi_uart_ForceTxBreak(sys_services_uart_hdevice, false);
        sys_services_break_active = false;
    }
    else
    {
        /* Start transmitting the break condition. */
        adi_uart_ForceTxBreak(sys_services_uart_hdevice, true);
        sys_services_break_active = true;
        sys_services_uart_break_time = cld_time_get();
        sys_services_uart_break_timeout = duration;
    }
}

/*=============================================================================
Function:       sys_services_uart_irq_callback

Parameters:     pHandle - ADI system services UART handle.
                u32Arg - The UART Event that occurred.
                pArg - Not used.

Description:    Function called by the ADI Systems Service UART driver when a
                UART interrupt occurred.

Returns:        None.
==============================================================================*/
static void sys_services_uart_irq_callback(void* pHandle, uint32_t u32Arg, void* pArg)
{
    CLD_BF70x_CDC_Serial_State cdc_serial_state =
    {
        .u.state = 0
    };
    ADI_UART_HANDLE pDevice = (ADI_UART_HANDLE)pHandle;
    ADI_UART_EVENT event    = (ADI_UART_EVENT)u32Arg;

    switch (event)
    {
        case ADI_UART_EVENT_TX_BUFFER_PROCESSED:
            sys_services_uart_tx_idle = true;

        break;
        case ADI_UART_EVENT_RX_BUFFER_PROCESSED:
            user_cdc_tx_serial_data(1, sys_services_uart_rx_buffer);
            adi_uart_SubmitRxBuffer(sys_services_uart_hdevice, sys_services_uart_rx_buffer, SYS_SERVICES_UART_RX_BUFFER_SIZE);
        break;

        case ADI_UART_EVENT_OVERRUN_ERROR:
            cdc_serial_state.u.bits.rx_data_overrun = 1;
            cld_bf70x_cdc_lib_send_serial_state(&cdc_serial_state);
        break;
        case ADI_UART_EVENT_PARITY_ERROR:
            cdc_serial_state.u.bits.parity_error = 1;
            cld_bf70x_cdc_lib_send_serial_state(&cdc_serial_state);
        break;
        case ADI_UART_EVENT_FRAMING_ERROR:
            cdc_serial_state.u.bits.framing_error = 1;
            cld_bf70x_cdc_lib_send_serial_state(&cdc_serial_state);
        break;
        case ADI_UART_EVENT_BREAK_INTERRUPT:
            cdc_serial_state.u.bits.break_detect = 1;
            cld_bf70x_cdc_lib_send_serial_state(&cdc_serial_state);
        break;
        case ADI_UART_EVENT_RX_FIFO_WM:
        case ADI_UART_EVENT_RECIEVED_ADDR:
        case ADI_UART_EVENT_AUTOBAUD_COMPLETE:
        case ADI_UART_EVENT_TX_COMPLETE:
        case ADI_UART_EVENT_RX_DMA_ERROR:
        case ADI_UART_EVENT_TX_DMA_ERROR:
        break;
        default:
        break;
    }
}
