/*==============================================================================
    FILE:           user_cdc.c

    DESCRIPTION:    Uses the cld_cdc_lib library to implement a CDC USB to Serial
                    device.

    Copyright (c) 2014 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/
#include "user_cdc.h"
#include "cld_bf70x_cdc_lib.h"

#include "sys_services_uart.h"

#include <ADSP-BF707_device.h>

/* Function Prototypes. */
static void user_cdc_usb_in_transfer_complete (void);
static void user_cdc_console_rx_byte (unsigned char byte);
static void user_cdc_usb_event (CLD_USB_Event event);

static CLD_USB_Data_Received_Return_Type user_cdc_serial_data_rx_transfer_complete (void);
static CLD_USB_Transfer_Request_Return_Type user_cdc_serial_data_received (CLD_USB_Transfer_Params * p_transfer_data);
static CLD_USB_Transfer_Request_Return_Type user_cdc_cmd_send_encapsulated_cmd (CLD_USB_Transfer_Params * p_transfer_data);
static CLD_USB_Transfer_Request_Return_Type user_cdc_cmd_get_encapsulated_resp (CLD_USB_Transfer_Params * p_transfer_data);

static CLD_USB_Data_Received_Return_Type user_cdc_cmd_set_line_coding (CLD_BF70x_CDC_Line_Coding * p_line_coding);
static CLD_RV user_cdc_cmd_get_line_coding (CLD_BF70x_CDC_Line_Coding * p_line_coding);
static CLD_USB_Data_Received_Return_Type user_cdc_cmd_set_control_line_state (CLD_BF70x_CDC_Control_Line_State * p_control_line_state);
static CLD_USB_Data_Received_Return_Type user_cdc_cmd_send_break (unsigned short duration);

/* CDC Notification Interrupt IN endpoint parameters. */
static CLD_BF70x_CDC_Notification_Endpoint_Params user_cdc_notification_ep_params =
{
    .endpoint_number                = 1,
    .max_packet_size_full_speed     = 64,
    .polling_interval_full_speed    = 1,
    .max_packet_size_high_speed     = 64,
    .polling_interval_high_speed    = 4,  /* 1ms */
};

/* CDC Serial Data Bulk OUT endpoint parameters. */
static CLD_Serial_Data_Bulk_Endpoint_Params user_cdc_serial_data_rx_ep_params =
{
    .endpoint_number                = 2,
    .max_packet_size_full_speed     = 64,
    .max_packet_size_high_speed     = 512,
};

/* CDC Serial Data Bulk IN endpoint parameters. */
static CLD_Serial_Data_Bulk_Endpoint_Params user_cdc_serial_data_tx_ep_params =
{
    .endpoint_number                = 2,
    .max_packet_size_full_speed     = 64,
    .max_packet_size_high_speed     = 512,
};

/* CLD CDC library initialization data. */
CLD_BF70x_CDC_Lib_Init_Params user_cdc_init_params =
{
    .timer_num  = CLD_TIMER_0,                          /* Timer used by the CLD Library */
    .uart_num   = CLD_UART_0,                           /* UART used by the CLD Library.
                                                            If the uart_num is set to
                                                            CLD_UART_DISABLE the CLD library
                                                            will not use a UART */
    .uart_baud  = 115200,                               /* CLD Library CONSOLE print UART
                                                           baudrate. */
    .sclk0      = 100000000u,                           /* Blackfin SCLK0 frequency */
    .fp_console_rx_byte = user_cdc_console_rx_byte,     /* Function called when a byte
                                                           is received by the CONSOLE
                                                           UART. */
    .vendor_id = 0x064b,                                /* Analog Devices Vendor ID */
    .product_id = 0x0003,                               /*  */

    /* Pointer to the serial data rx bulk endpoint parameters. */
    .p_serial_data_rx_endpoint_params = &user_cdc_serial_data_rx_ep_params,
    /* Pointer to the serial data tx bulk endpoint parameters. */
    .p_serial_data_tx_endpoint_params = &user_cdc_serial_data_tx_ep_params,
    /* Pointer to the CDC notification endpoint parameters. */
    .p_notification_endpoint_params = &user_cdc_notification_ep_params,

    /* Function called when serial data is received. */
    .fp_serial_data_received = user_cdc_serial_data_received,
    /* Function called when a CDC Send Encapsulated Command request is received */
    .fp_cdc_cmd_send_encapsulated_cmd   = user_cdc_cmd_send_encapsulated_cmd,
    /* Function called when a CDC Get Encapsulated Command request is received */
    .fp_cdc_cmd_get_encapsulated_resp   = user_cdc_cmd_get_encapsulated_resp,
    /* Function called when a CDC Set Line Coding request is received */
    .fp_cdc_cmd_set_line_coding         = user_cdc_cmd_set_line_coding,
    /* Function called when a CDC Get Line Coding request is received */
    .fp_cdc_cmd_get_line_coding         = user_cdc_cmd_get_line_coding,
    /* Function called when a CDC Set Control Line request is received */
    .fp_cdc_cmd_set_control_line_state  = user_cdc_cmd_set_control_line_state,
    /* Function called when a CDC Send Break request is received */
    .fp_cdc_cmd_send_break              = user_cdc_cmd_send_break,

    .usb_bus_max_power = 0,                             /* The ADSP-BF707 EZ-Board is
                                                           self-powered (not USB bus-powered) */

    .support_cdc_network_connection     = 1,            /* 1 = Network Notification supported
                                                           0 = Network Notification not supported */

    .cdc_class_bcd_version              = 0x0120,       /* CDC Version 1.2 */
    .cdc_class_control_protocol_code    = 0,            /* No Class Specific protocol */

    .device_descriptor_bcdDevice        = 0x0100,      /* Set USB Device Descriptor
                                                           firmware version to 1.00 */

    /* USB string descriptors - Set to NULL if not required */
    .p_usb_string_manufacturer                  = "Analog Devices Inc",
    .p_usb_string_product                       = "BF707 CDC UART",
    .p_usb_string_serial_number                 = "00000001",
    .p_usb_string_configuration                 = CLD_NULL,
    .p_usb_string_communication_class_interface = "CLD CDC Ctrl",
    .p_usb_string_data_class_interface          = "CLD CDC Data",
    .usb_string_language_id     = 0x0409,               /* English (US) language ID */

    /* Function called when one of the following USB events occurs
        CLD_USB_CABLE_CONNECTED      - USB cable connected
        CLD_USB_CABLE_DISCONNECTED   - USB cable disconnected
        CLD_USB_ENUMERATED_CONFIGURED- USB Enumerated (USB Configuration set to non-zero value)
        CLD_USB_UN_CONFIGURED        - USB Configuration set to 0.
        CLD_USB_BUS_RESET            - USB Bus Reset received. */
    .fp_cld_usb_event_callback = user_cdc_usb_event,
};

/* Current CDC line coding settings (default to 115.2k 8 bits, 1 Stop bit, no parity) */
static CLD_BF70x_CDC_Line_Coding user_cdc_line_coding =
{
    .data_terminal_rate = 115200,
    .num_stop_bits = CLD_BF70x_CDC_LINE_CODING_1_STOP_BITS,
    .parity = CLD_BF70x_CDC_LINE_CODING_PARITY_NONE,
    .num_data_bits = 8,
};

/* Variables used to receive serial data from the CDC USB Host. */
#define USER_CDC_SERIAL_BUFFER_SIZE     512
static unsigned short user_cdc_serial_data_num_rx_bytes;
static unsigned char user_cdc_serial_data_rx_buffer[USER_CDC_SERIAL_BUFFER_SIZE];

/* Set to true if serial data is received while the ADI system services UART
   is still transmitting the previously received data. */
static bool user_cdc_serial_data_rx_paused = false;

/*=============================================================================
Function:       user_cdc_init

Parameters:     None.

Description:    Initializes the CLD CDC library.

Returns:        USER_CDC_INIT_SUCCESS/USER_CDC_INIT_ONGOING/USER_CDC_INIT_FAILED.
==============================================================================*/
User_CDC_Init_Return_Code user_cdc_init (void)
{
    static unsigned char user_init_state = 0;
    CLD_RV cld_rv = CLD_ONGOING;

    /* Initialize the CLD CDC Library */
    cld_rv = cld_bf70x_cdc_lib_init(&user_cdc_init_params);

    if (cld_rv == CLD_SUCCESS)
    {
        /* Initialize the system services uart and set the line coding to the
           default value */
        sys_services_uart_init();
        sys_services_uart_set_line_coding(&user_cdc_line_coding);

        /* Connect to the USB Host */
        cld_lib_usb_connect();

        return USER_CDC_INIT_SUCCESS;
    }
    else if (cld_rv == CLD_FAIL)
    {
        return USER_CDC_INIT_FAILED;
    }
    else
    {
        return USER_CDC_INIT_ONGOING;
    }
}

/*=============================================================================
Function:       user_cdc_main

Parameters:     None.

Description:    Example user mainline.

Returns:        None.
==============================================================================*/
void user_cdc_main (void)
{
    #define MINUTES(x)      (x/60000)
    #define SECONDS(x)      ((x/1000)%60)
    #define M_SECONDS(x)    (x%1000)

    static CLD_Time main_time = 0;
    static CLD_Time msg_time = 0;
    static CLD_Time run_time = 0;

    cld_bf70x_cdc_lib_main();

    if (cld_time_passed_ms(main_time) >= 250u)
    {
        main_time = cld_time_get();
        pADI_PORTA->DATA_TGL = (1 << 0);
    }
    if (cld_time_passed_ms(msg_time) >= 237u)
    {
        msg_time = cld_time_get();
        run_time = cld_time_passed_ms(0);
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Run time: %2.2d:%2.2d:%3.3d\r",
            MINUTES(run_time),
            SECONDS(run_time),
            M_SECONDS(run_time));
    }

    /* If a serial data Bulk OUT request was paused and the system services UART
       is now idle, resume the Bulk OUT transfer. */
    if (user_cdc_serial_data_rx_paused == true)
    {
        if (sys_services_uart_tx_idle == true)
        {
            user_cdc_serial_data_rx_paused = false;
            cld_bf70x_cdc_lib_resume_paused_serial_data_transfer();
        }
    }
    sys_services_uart_main();
}

/*=============================================================================
Function:       user_cdc_usb_in_transfer_complete

Parameters:     None.

Description:    This function is called by the CLD CDC library when Bulk IN
                serial data has been transmitted to the Host.

Returns:        None.
==============================================================================*/
static void user_cdc_usb_in_transfer_complete (void)
{
    /* The serial data has been send to the Host. */
    pADI_PORTB->DATA_TGL = (1 << 1);
}

/*=============================================================================
Function:       user_cdc_serial_data_rx_transfer_complete

Parameters:     None.

Description:    This function is called by the CLD CDC library when the serial data
                has been received on the Bulk OUT endpoint.

Returns:        CLD_USB_DATA_GOOD - The received data is valid.
==============================================================================*/
static CLD_USB_Data_Received_Return_Type user_cdc_serial_data_rx_transfer_complete (void)
{
    unsigned char i;

    pADI_PORTB->DATA_TGL = (1 << 1);

    /* Output the received serial data using the CLD Console. */
    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Rx serial data:");
    for (i = 0; i < user_cdc_serial_data_num_rx_bytes; i++)
    {
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "%c", user_cdc_serial_data_rx_buffer[i]);
    }
    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r");

    /* Transmit the received data using the Blackfin UART. */
    sys_services_uart_tx_buffer(user_cdc_serial_data_num_rx_bytes, user_cdc_serial_data_rx_buffer);

    return CLD_USB_DATA_GOOD;
}

/*=============================================================================
Function:       user_cdc_serial_data_received

Parameters:     p_transfer_data - Pointer to the bulk OUT transfer data.
                    p_transfer_data->num_bytes     - Number of received bulk OUT bytes.
                                                     This value can be set to the total
                                                     transfer size if more then one
                                                     packet is expected.
                    p_transfer_data->p_data_buffer - Set to the address where the
                                                     bulk data should be written.
                    p_transfer_data->callback.usb_out_transfer_complete -
                                                     Function called when the
                                                     requested received bytes have
                                                     been written to p_data_buffer.
                    p_transfer_data->transfer_aborted_callback -
                                                     Optional function that is
                                                     called if the transfer is
                                                     aborted.
                    p_transfer_data->transfer_timeout_ms -
                                                     Bulk transfer timeout in
                                                     milliseconds.

Description:    This function is called by the CLD CDC library when data is
                received on the Bulk OUT endpoint. This function sets the
                p_transfer_data parameters to select where the received data
                should be stored, and what function should be called when the
                transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Store the bulk data using the p_transfer_data
                                          parameters.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         bulk out packet so pause the transfer
                                         until the cld_bulk_resume_paused_bulk_out_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Discard this bulk packet.
                CLD_USB_TRANSFER_STALL - Stall the bulk OUT endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_cdc_serial_data_received (CLD_USB_Transfer_Params * p_transfer_data)
{
    if (sys_services_uart_tx_idle == true)
    {
        p_transfer_data->p_data_buffer = (unsigned char *)&user_cdc_serial_data_rx_buffer;
        p_transfer_data->callback.fp_usb_out_transfer_complete = user_cdc_serial_data_rx_transfer_complete;
        p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
        p_transfer_data->transfer_timeout_ms = 1000;

        user_cdc_serial_data_num_rx_bytes = p_transfer_data->num_bytes;
        return CLD_USB_TRANSFER_ACCEPT;
    }
    else
    {
        user_cdc_serial_data_rx_paused = true;
        return CLD_USB_TRANSFER_PAUSE;
    }
}

/*=============================================================================
Function:       user_cdc_cmd_send_encapsulated_cmd

Parameters:     p_transfer_data - Pointer to the Control OUT transfer data.
                    p_transfer_data->num_bytes     - Number of Control OUT bytes.
                    p_transfer_data->p_data_buffer - Set to the address where to
                                                     store the Control OUT data.
                    p_transfer_data->callback.usb_in_transfer_complete -
                                                     Function called when the
                                                     Control OUT bytes have been
                                                     received.
                    p_transfer_data->transfer_aborted_callback -
                                                     Optional function that is
                                                     called if the transfer is
                                                     aborted.

Description:    This function is called by the CLD CDC library when the CDC Send
                Encapsulated Command request is received. This function sets the
                p_transfer_data parameters to select where the received data
                should be stored, and what function should be called when the
                transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Store the data using the p_transfer_data
                                          parameters.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         Control OUT request so pause the transfer
                                         until the cld_cdc_lib_resume_paused_control_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Discard the received data.
                CLD_USB_TRANSFER_STALL - Stall the Control endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_cdc_cmd_send_encapsulated_cmd (CLD_USB_Transfer_Params * p_transfer_data)
{
    /* Not currently supported by this example. */
    return CLD_USB_TRANSFER_STALL;
}

/*=============================================================================
Function:       user_cdc_cmd_get_encapsulated_resp

Parameters:     p_transfer_data - Pointer to the Control IN transfer data.
                    p_transfer_data->num_bytes     - Number of Control IN bytes.
                    p_transfer_data->p_data_buffer - Set to the address where to
                                                     source the Control IN data.
                    p_transfer_data->callback.usb_in_transfer_complete -
                                                     Function called when the
                                                     Control IN bytes have been
                                                     sent to the Host.
                    p_transfer_data->transfer_aborted_callback -
                                                     Optional function that is
                                                     called if the transfer is
                                                     aborted.

Description:    This function is called by the CLD CDC library when a CDC Get
                Encapsulated Command request is received. This function sets the
                p_transfer_data parameters to return the requested data to the Host,
                and what function should be called when the transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Transmit the specified data to the Host.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         Control IN request so pause the transfer
                                         until the cld_cdc_lib_resume_paused_control_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Return a zero length packet to the Host.
                CLD_USB_TRANSFER_STALL - Stall the Control endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_cdc_cmd_get_encapsulated_resp (CLD_USB_Transfer_Params * p_transfer_data)
{
    /* Not currently supported by this example. */
    return CLD_USB_TRANSFER_STALL;
}

/*=============================================================================
Function:       user_cdc_cmd_set_line_coding

Parameters:     p_line_coding - Pointer to received CLD_CDC_Line_Coding structure.

Description:    This function is called by the CLD CDC library when a CDC Set Line Coding
                request is received. This function saves the requested line coding
                and configures the Blackfin UART accordingly.

Returns:        CLD_USB_DATA_GOOD - Set line coding parameters accepted.
                CLD_USB_DATA_BAD_STALL - An error occurred while applying the requested
                                        set line coding parameters.
==============================================================================*/
static CLD_USB_Data_Received_Return_Type user_cdc_cmd_set_line_coding (CLD_BF70x_CDC_Line_Coding * p_line_coding)
{
    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Set Line Coding\n\r");
    user_cdc_line_coding = *p_line_coding;
    return (sys_services_uart_set_line_coding(&user_cdc_line_coding) == CLD_SUCCESS ? CLD_USB_DATA_GOOD : CLD_USB_DATA_BAD_STALL);
}

/*=============================================================================
Function:       user_cdc_cmd_get_line_coding

Parameters:     p_line_coding - Pointer to store the current CLD_CDC_Line_Coding
                                structure.

Description:    This function is called by the CLD CDC library when a CDC Get Line Coding
                request is received. This loads the current line coding
                into the p_line_coding structure passed to the function.

Returns:        CLD_SUCCESS - There was no errors copying the current line coding.
==============================================================================*/
static CLD_RV user_cdc_cmd_get_line_coding (CLD_BF70x_CDC_Line_Coding * p_line_coding)
{
    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Get Line Coding\n\r");
    *p_line_coding = user_cdc_line_coding;

    return CLD_SUCCESS;
}

/*=============================================================================
Function:       user_cdc_cmd_set_control_line_state

Parameters:     p_control_line_state - Pointer to received CLD_CDC_Control_Line_State
                                       structure.

Description:    This function is called by the CLD CDC library when a CDC Set
                Control Line State request is received.

Returns:        CLD_USB_DATA_GOOD - Set control line state parameters accepted.
                CLD_USB_DATA_BAD_STALL - An error occurred while applying the requested
                                        set control line state parameters.
==============================================================================*/
static CLD_USB_Data_Received_Return_Type user_cdc_cmd_set_control_line_state (CLD_BF70x_CDC_Control_Line_State * p_control_line_state)
{
    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Set Line Control State\n\r");
    return CLD_USB_DATA_GOOD;
}

/*=============================================================================
Function:       user_cdc_cmd_send_break

Parameters:     duration - Requested Break duration in milliseconds.

Description:    This function is called by the CLD CDC library when a CDC Send
                Break request is received.

Returns:        CLD_USB_DATA_GOOD - Set control line state parameters accepted.
                CLD_USB_DATA_BAD_STALL - An error occurred while applying the requested
                                        set control line state parameters.
==============================================================================*/
static CLD_USB_Data_Received_Return_Type user_cdc_cmd_send_break (unsigned short duration)
{
    sys_services_uart_send_break(duration);
    return CLD_USB_DATA_GOOD;
}

/*=============================================================================
Function:       user_cdc_tx_serial_data

Parameters:     length - number of bytes to transmit.
                p_buffer - pointer to the data buffer to transmit

Description:    Transmits the specified data to the CDC Host using the serial data
                Bulk IN endpoint.

Returns:        None.
==============================================================================*/
void user_cdc_tx_serial_data (unsigned char length, unsigned char * p_buffer)
{
    static CLD_USB_Transfer_Params transfer_params;
    transfer_params.num_bytes = length;
    transfer_params.p_data_buffer = p_buffer;
    transfer_params.callback.fp_usb_in_transfer_complete = user_cdc_usb_in_transfer_complete;
    transfer_params.fp_transfer_aborted_callback = CLD_NULL;
    transfer_params.transfer_timeout_ms = 1000;

    cld_bf70x_cdc_lib_transmit_serial_data (&transfer_params);
}

/*=============================================================================
Function:       user_cdc_console_rx_byte

Parameters:     byte - received byte.

Description:    Function called when the cld_console UART receives a byte.

Returns:        None.
==============================================================================*/
static void user_cdc_console_rx_byte (unsigned char byte)
{
    static const CLD_CONSOLE_COLOR cld_console_colors[CLD_CONSOLE_NUM_COLORS] =
    {
        CLD_CONSOLE_RED,
        CLD_CONSOLE_GREEN,
        CLD_CONSOLE_YELLOW,
        CLD_CONSOLE_BLUE,
        CLD_CONSOLE_PURPLE,
        CLD_CONSOLE_CYAN,
        CLD_CONSOLE_WHITE,
        CLD_CONSOLE_BLACK,
    };
    static const char * color_strings[CLD_CONSOLE_NUM_COLORS] =
    {
        "Red",
        "Green",
        "Yellow",
        "Blue",
        "Purple",
        "Cyan",
        "White",
        "Black",
    };
    unsigned char i;

    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\r\nUser received %c\r\n", byte);

    if (byte == '!')
    {
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r----- CLD Console Color Demo -----\n\r\n\r");
        for (i = 0; i < CLD_CONSOLE_NUM_COLORS; i++)
        {
            cld_console(cld_console_colors[i], cld_console_colors[CLD_CONSOLE_NUM_COLORS - i - 1], "%s text with a %s background\n\r",
                        color_strings[i], color_strings[CLD_CONSOLE_NUM_COLORS - i - 1]);
        }
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r----------------------------------\n\r\n\r");
    }
}

/*=============================================================================
Function:       user_cdc_usb_event

Parameters:     event - identifies which USB event has occurred.

Description:    Function Called when a USB event occurs.

Returns:        None.
==============================================================================*/
static void user_cdc_usb_event (CLD_USB_Event event)
{
    switch (event)
    {
        case CLD_USB_CABLE_CONNECTED:
            pADI_PORTA->DATA_SET = (1 << 1);
        break;
        case CLD_USB_CABLE_DISCONNECTED:
            pADI_PORTA->DATA_CLR = (1 << 1);
        break;
        case CLD_USB_ENUMERATED_CONFIGURED:
            cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "CLD CDC Device Enumerated\n\r");
        break;
        case CLD_USB_UN_CONFIGURED:
        break;
        case CLD_USB_BUS_RESET:
        break;
    }
}
